@@ -39,6 +39,11 @@ class AuthenticationFailure(Exception):
         self.message = message
 
 
+class AuthorizationFailure(Exception):
+    def __init__(self, message):
+        self.message = message
+
+
 class Job:
     def __init__(self, job_id):
         self._job_id = job_id
@@ -81,13 +86,16 @@ class InputData:
                 self.data["operator"] = {
                     "pauli-terms": pauli_terms,
                     "coefficients": coefficients,
-                    "operator-string-representation": str(operator)
+                    "operator-string-representation": str(operator),
                 }
             elif label == "pub":
                 content = self.validate_and_serialize_pub(content)
                 if not "pubs" in self.data.keys():
                     self.data["pubs"] = []
                 self.data["pubs"].append(content)
+            elif label == "molecule-info`":
+                self.validate_molecule_info(content)
+                self.data[label] = content
             else:
                 self.data[label] = content
         except (OverflowError, TypeError, ValueError):
@@ -114,6 +122,28 @@ class InputData:
                 f"An input data item of type '{label}' has already been added to the job input data. Multiple data items of same category are allowed only for PUBs."
             )
 
+    def validate_molecule_info(self, molecule_info):
+        if "mutiplicity" in molecule_info and not isinstance(
+            molecule_info["mutiplicity"], int
+        ):
+            raise Exception("Multiplicity must be an integer.")
+        if "charge" in molecule_info and not isinstance(molecule_info["charge"], int):
+            raise Exception("Charge must be an integer.")
+        if (
+            "units" in molecule_info
+            and molecule_info["units"].lower() != "angstrom"
+            and molecule_info["units"].lower() != "bohr"
+        ):
+            raise Exception(
+                "The units of the molecule coordinates must be either 'Angstrom' or 'Bohr'."
+            )
+        if "masses" in molecule_info and not all(
+            isinstance(m, (int, float)) for m in molecule_info["masses"]
+        ):
+            raise Exception(
+                "Masses must be a list of floats, one for each nucleus in the molecule."
+            )
+
     def validate_and_serialize_pub(self, pub):
         shots = None
         paramaters = None
@@ -293,7 +323,11 @@ In case the service has been recently started please wait 5 minutes for it to be
                 time.time() + token_response["refresh_expires_in"] - 5
             )  # seconds
         except AuthenticationFailure as ex:
-            print("Failed to authenticate with the quantum provider.")
+            print(ex.message)
+        except AuthorizationFailure as ex:
+            print(
+                "Failed to authenticate with the quantum provider. Make sure you are using the correct email account."
+            )
             if self._debug:
                 print("More details: ", ex.message)
         except Exception as ex:
@@ -586,6 +620,15 @@ In case the service has been recently started please wait 5 minutes for it to be
         except requests.exceptions.RequestException as e:
             return False
 
+    def _is_server_under_maintenance(self, url):
+        try:
+            response = requests.get(url, verify=self._use_https)
+            if "Under Maintenance" in response.text:
+                return True
+            return False
+        except requests.exceptions.RequestException as e:
+            return False
+
     def _store_state(self):
         state = str(uuid.uuid4())
         session = requests.Session()
@@ -597,7 +640,11 @@ In case the service has been recently started please wait 5 minutes for it to be
         if session_response.status_code != 200:
             if not self._is_server_online(self._asp_net_url):
                 raise AuthenticationFailure(
-                    f"The service you are trying to access at: {self._asp_net_url} is not responding."
+                    f"The service you are trying to access at: {self._asp_net_url} is not online."
+                )
+            elif self._is_server_under_maintenance(self._asp_net_url):
+                raise AuthenticationFailure(
+                    f"The service you are trying to access at: {self._asp_net_url} is under maintenance."
                 )
             else:
                 raise AuthenticationFailure(
@@ -637,9 +684,10 @@ In case the service has been recently started please wait 5 minutes for it to be
                     params={"state": self._state},
                     verify=self._use_https,
                 )
+                # TODO: what if I use a wrong email account
                 if response.status_code == 400:
                     if response.text == "Authorization state is missing.":
-                        raise AuthenticationFailure(
+                        raise AuthorizationFailure(
                             "Authorization state not found on remote server."
                         )
                     continue
@@ -647,11 +695,11 @@ In case the service has been recently started please wait 5 minutes for it to be
                 auth_code = data.split(": ")[1]
                 return auth_code
             except requests.RequestException as e:
-                raise AuthenticationFailure(
+                raise AuthorizationFailure(
                     f"Remote server is not responding to attempts to retrieve authorization code, exception is {e}."
                 )
 
-        raise AuthenticationFailure("Authorization code not received")
+        raise AuthorizationFailure("Authorization code not received.")
 
     def is_valid_uuid(self, value: str) -> bool:
         try:
